# 1 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
!--------------------------------------------------------------------------------------------------!
!   CP2K: A general program to perform molecular dynamics simulations                              !
!   Copyright 2000-2020 CP2K developers group <https://cp2k.org>                                   !
!                                                                                                  !
!   SPDX-License-Identifier: GPL-2.0-or-later                                                      !
!--------------------------------------------------------------------------------------------------!

! **************************************************************************************************
!> \brief Utility routines for the memory handling.
!> \par History
!>      (12.2017) remove stop_memory
!> \author Matthias Krack (25.06.1999)
! **************************************************************************************************
MODULE memory_utilities

   USE kinds, ONLY: default_path_length, &
                    default_string_length, &
                    dp, int_8
#include "../base/base_uses.f90"

   IMPLICIT NONE

   PRIVATE

   CHARACTER(len=*), PARAMETER, PRIVATE :: moduleN = 'memory_utilities'

   PUBLIC :: reallocate

   INTERFACE reallocate
      MODULE PROCEDURE reallocate_c1, reallocate_c2, reallocate_c3, reallocate_c4, &
         reallocate_i1, reallocate_i2, reallocate_i3, reallocate_i4, &
         reallocate_r1, reallocate_r2, reallocate_r3, reallocate_r4, &
         reallocate_r5, reallocate_s1, reallocate_l1, reallocate_8i1, &
         reallocate_8i2
   END INTERFACE

CONTAINS

# 43 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
# 78 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

! **************************************************************************************************
!> \brief (Re)Allocate a complex vector with a new dimension
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \par History
!>      none
!> \author Matthias Krack (28.11.2005,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_c1(p, lb1_new, ub1_new)

      COMPLEX(KIND=dp), DIMENSION(:), POINTER            :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new

      COMPLEX(KIND=dp), PARAMETER                        :: zero = (0.0_dp, 0.0_dp)

      COMPLEX(KIND=dp), ALLOCATABLE, DIMENSION(:)        :: work


# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb1, lb1_old, ub1, ub1_old
# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p)) THEN
# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1_old = LBOUND(p, 1)
# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1_old = UBOUND(p, 1)
# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1 = MAX(lb1_new, lb1_old)
# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1 = MIN(ub1_new, ub1_old)
# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ALLOCATE (work(lb1:ub1))
# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      work(lb1:ub1) = p(lb1:ub1)
# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (p)
# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   ALLOCATE (p(lb1_new:ub1_new))
# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   p = zero
# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p) .AND. ALLOCATED(work)) THEN
# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      p(lb1:ub1) = work(lb1:ub1)
# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (work)
# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 97 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_c1

! **************************************************************************************************
!> \brief (Re)Allocate a complex array 2D with a new dimension
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \par History
!>      none
!> \author Matthias Krack (28.11.2005,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_c2(p, lb1_new, ub1_new, lb2_new, ub2_new)

      COMPLEX(KIND=dp), DIMENSION(:, :), POINTER         :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new

      COMPLEX(KIND=dp), PARAMETER                        :: zero = (0.0_dp, 0.0_dp)

      COMPLEX(KIND=dp), ALLOCATABLE, DIMENSION(:, :)     :: work


# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb1, lb1_old, ub1, ub1_old
# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb2, lb2_old, ub2, ub2_old
# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p)) THEN
# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1_old = LBOUND(p, 1)
# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1_old = UBOUND(p, 1)
# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1 = MAX(lb1_new, lb1_old)
# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1 = MIN(ub1_new, ub1_old)
# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2_old = LBOUND(p, 2)
# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2_old = UBOUND(p, 2)
# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2 = MAX(lb2_new, lb2_old)
# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2 = MIN(ub2_new, ub2_old)
# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ALLOCATE (work(lb1:ub1,lb2:ub2))
# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      work(lb1:ub1,lb2:ub2) = p(lb1:ub1,lb2:ub2)
# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (p)
# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new))
# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   p = zero
# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p) .AND. ALLOCATED(work)) THEN
# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      p(lb1:ub1,lb2:ub2) = work(lb1:ub1,lb2:ub2)
# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (work)
# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 121 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_c2

! **************************************************************************************************
!> \brief (Re)Allocate a complex array 3D with a new dimension
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \param lb3_new ...
!> \param ub3_new ...
!> \par History
!>      none
!> \author Matthias Krack (28.11.2005,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_c3(p, lb1_new, ub1_new, lb2_new, ub2_new, lb3_new, ub3_new)

      COMPLEX(KIND=dp), DIMENSION(:, :, :), POINTER      :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new, &
                                                            lb3_new, ub3_new

      COMPLEX(KIND=dp), PARAMETER                        :: zero = (0.0_dp, 0.0_dp)

      COMPLEX(KIND=dp), ALLOCATABLE, DIMENSION(:, :, :)  :: work


# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb1, lb1_old, ub1, ub1_old
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb2, lb2_old, ub2, ub2_old
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb3, lb3_old, ub3, ub3_old
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p)) THEN
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1_old = LBOUND(p, 1)
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1_old = UBOUND(p, 1)
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1 = MAX(lb1_new, lb1_old)
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1 = MIN(ub1_new, ub1_old)
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2_old = LBOUND(p, 2)
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2_old = UBOUND(p, 2)
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2 = MAX(lb2_new, lb2_old)
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2 = MIN(ub2_new, ub2_old)
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb3_old = LBOUND(p, 3)
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub3_old = UBOUND(p, 3)
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb3 = MAX(lb3_new, lb3_old)
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub3 = MIN(ub3_new, ub3_old)
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ALLOCATE (work(lb1:ub1,lb2:ub2,lb3:ub3))
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      work(lb1:ub1,lb2:ub2,lb3:ub3) = p(lb1:ub1,lb2:ub2,lb3:ub3)
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (p)
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new,lb3_new:ub3_new))
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   p = zero
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p) .AND. ALLOCATED(work)) THEN
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      p(lb1:ub1,lb2:ub2,lb3:ub3) = work(lb1:ub1,lb2:ub2,lb3:ub3)
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (work)
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 148 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_c3

! **************************************************************************************************
!> \brief (Re)Allocate a complex array 4D with a new dimension
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \param lb3_new ...
!> \param ub3_new ...
!> \param lb4_new ...
!> \param ub4_new ...
!> \par History
!>      none
!> \author Matthias Krack (28.11.2005,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_c4(p, lb1_new, ub1_new, lb2_new, ub2_new, lb3_new, ub3_new, &
                            lb4_new, ub4_new)

      COMPLEX(KIND=dp), DIMENSION(:, :, :, :), POINTER   :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new, &
                                                            lb3_new, ub3_new, lb4_new, ub4_new

      COMPLEX(KIND=dp), PARAMETER                        :: zero = (0.0_dp, 0.0_dp)

      COMPLEX(KIND=dp), ALLOCATABLE, &
         DIMENSION(:, :, :, :)                           :: work


# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb1, lb1_old, ub1, ub1_old
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb2, lb2_old, ub2, ub2_old
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb3, lb3_old, ub3, ub3_old
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb4, lb4_old, ub4, ub4_old
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p)) THEN
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1_old = LBOUND(p, 1)
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1_old = UBOUND(p, 1)
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1 = MAX(lb1_new, lb1_old)
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1 = MIN(ub1_new, ub1_old)
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2_old = LBOUND(p, 2)
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2_old = UBOUND(p, 2)
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2 = MAX(lb2_new, lb2_old)
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2 = MIN(ub2_new, ub2_old)
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb3_old = LBOUND(p, 3)
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub3_old = UBOUND(p, 3)
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb3 = MAX(lb3_new, lb3_old)
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub3 = MIN(ub3_new, ub3_old)
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb4_old = LBOUND(p, 4)
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub4_old = UBOUND(p, 4)
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb4 = MAX(lb4_new, lb4_old)
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub4 = MIN(ub4_new, ub4_old)
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ALLOCATE (work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4))
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4) = p(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4)
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (p)
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new,lb3_new:ub3_new,lb4_new:ub4_new))
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   p = zero
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p) .AND. ALLOCATED(work)) THEN
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      p(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4) = work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4)
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (work)
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 179 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_c4

! **************************************************************************************************
!> \brief (Re)Allocate an integer vector with a new dimension.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_i1(p, lb1_new, ub1_new)

      INTEGER, DIMENSION(:), POINTER                     :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new

      INTEGER, PARAMETER                                 :: zero = 0

      INTEGER, ALLOCATABLE, DIMENSION(:)                 :: work


# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb1, lb1_old, ub1, ub1_old
# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p)) THEN
# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1_old = LBOUND(p, 1)
# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1_old = UBOUND(p, 1)
# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1 = MAX(lb1_new, lb1_old)
# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1 = MIN(ub1_new, ub1_old)
# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ALLOCATE (work(lb1:ub1))
# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      work(lb1:ub1) = p(lb1:ub1)
# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (p)
# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   ALLOCATE (p(lb1_new:ub1_new))
# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   p = zero
# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p) .AND. ALLOCATED(work)) THEN
# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      p(lb1:ub1) = work(lb1:ub1)
# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (work)
# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 201 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_i1

! **************************************************************************************************
!> \brief (Re)Allocate an integer array with a new dimension.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_i2(p, lb1_new, ub1_new, lb2_new, ub2_new)

      INTEGER, DIMENSION(:, :), POINTER                  :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new

      INTEGER, PARAMETER                                 :: zero = 0

      INTEGER, ALLOCATABLE, DIMENSION(:, :)              :: work


# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb1, lb1_old, ub1, ub1_old
# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb2, lb2_old, ub2, ub2_old
# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p)) THEN
# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1_old = LBOUND(p, 1)
# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1_old = UBOUND(p, 1)
# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1 = MAX(lb1_new, lb1_old)
# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1 = MIN(ub1_new, ub1_old)
# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2_old = LBOUND(p, 2)
# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2_old = UBOUND(p, 2)
# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2 = MAX(lb2_new, lb2_old)
# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2 = MIN(ub2_new, ub2_old)
# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ALLOCATE (work(lb1:ub1,lb2:ub2))
# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      work(lb1:ub1,lb2:ub2) = p(lb1:ub1,lb2:ub2)
# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (p)
# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new))
# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   p = zero
# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p) .AND. ALLOCATED(work)) THEN
# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      p(lb1:ub1,lb2:ub2) = work(lb1:ub1,lb2:ub2)
# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (work)
# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 225 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_i2

! **************************************************************************************************
!> \brief (Re)Allocate an integer array 3D with a new dimension.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \param lb3_new ...
!> \param ub3_new ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_i3(p, lb1_new, ub1_new, lb2_new, ub2_new, lb3_new, ub3_new)

      INTEGER, DIMENSION(:, :, :), POINTER               :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new, &
                                                            lb3_new, ub3_new

      INTEGER, PARAMETER                                 :: zero = 0

      INTEGER, ALLOCATABLE, DIMENSION(:, :, :)           :: work


# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb1, lb1_old, ub1, ub1_old
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb2, lb2_old, ub2, ub2_old
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb3, lb3_old, ub3, ub3_old
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p)) THEN
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1_old = LBOUND(p, 1)
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1_old = UBOUND(p, 1)
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1 = MAX(lb1_new, lb1_old)
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1 = MIN(ub1_new, ub1_old)
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2_old = LBOUND(p, 2)
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2_old = UBOUND(p, 2)
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2 = MAX(lb2_new, lb2_old)
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2 = MIN(ub2_new, ub2_old)
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb3_old = LBOUND(p, 3)
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub3_old = UBOUND(p, 3)
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb3 = MAX(lb3_new, lb3_old)
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub3 = MIN(ub3_new, ub3_old)
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ALLOCATE (work(lb1:ub1,lb2:ub2,lb3:ub3))
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      work(lb1:ub1,lb2:ub2,lb3:ub3) = p(lb1:ub1,lb2:ub2,lb3:ub3)
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (p)
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new,lb3_new:ub3_new))
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   p = zero
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p) .AND. ALLOCATED(work)) THEN
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      p(lb1:ub1,lb2:ub2,lb3:ub3) = work(lb1:ub1,lb2:ub2,lb3:ub3)
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (work)
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 252 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_i3

! **************************************************************************************************
!> \brief (Re)Allocate an integer array 3D with a new dimension.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \param lb3_new ...
!> \param ub3_new ...
!> \param lb4_new ...
!> \param ub4_new ...
!> \par History
!>      none
!> \author Matthias Krack (04.10.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_i4(p, lb1_new, ub1_new, lb2_new, ub2_new, lb3_new, ub3_new, &
                            lb4_new, ub4_new)

      INTEGER, DIMENSION(:, :, :, :), POINTER            :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new, &
                                                            lb3_new, ub3_new, lb4_new, ub4_new

      INTEGER, PARAMETER                                 :: zero = 0

      INTEGER, ALLOCATABLE, DIMENSION(:, :, :, :)        :: work


# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb1, lb1_old, ub1, ub1_old
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb2, lb2_old, ub2, ub2_old
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb3, lb3_old, ub3, ub3_old
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb4, lb4_old, ub4, ub4_old
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p)) THEN
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1_old = LBOUND(p, 1)
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1_old = UBOUND(p, 1)
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1 = MAX(lb1_new, lb1_old)
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1 = MIN(ub1_new, ub1_old)
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2_old = LBOUND(p, 2)
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2_old = UBOUND(p, 2)
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2 = MAX(lb2_new, lb2_old)
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2 = MIN(ub2_new, ub2_old)
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb3_old = LBOUND(p, 3)
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub3_old = UBOUND(p, 3)
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb3 = MAX(lb3_new, lb3_old)
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub3 = MIN(ub3_new, ub3_old)
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb4_old = LBOUND(p, 4)
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub4_old = UBOUND(p, 4)
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb4 = MAX(lb4_new, lb4_old)
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub4 = MIN(ub4_new, ub4_old)
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ALLOCATE (work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4))
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4) = p(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4)
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (p)
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new,lb3_new:ub3_new,lb4_new:ub4_new))
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   p = zero
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p) .AND. ALLOCATED(work)) THEN
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      p(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4) = work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4)
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (work)
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 282 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_i4

! **************************************************************************************************
!> \brief (Re)Allocate an integer (int_8) vector with a new dimension.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_8i1(p, lb1_new, ub1_new)

      INTEGER(KIND=int_8), DIMENSION(:), POINTER         :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new

      INTEGER(KIND=int_8), PARAMETER                     :: zero = 0

      INTEGER(KIND=int_8), ALLOCATABLE, DIMENSION(:)     :: work


# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb1, lb1_old, ub1, ub1_old
# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p)) THEN
# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1_old = LBOUND(p, 1)
# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1_old = UBOUND(p, 1)
# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1 = MAX(lb1_new, lb1_old)
# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1 = MIN(ub1_new, ub1_old)
# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ALLOCATE (work(lb1:ub1))
# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      work(lb1:ub1) = p(lb1:ub1)
# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (p)
# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   ALLOCATE (p(lb1_new:ub1_new))
# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   p = zero
# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p) .AND. ALLOCATED(work)) THEN
# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      p(lb1:ub1) = work(lb1:ub1)
# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (work)
# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 304 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_8i1

! **************************************************************************************************
!> \brief (Re)Allocate an integer (int_8) array with a new dimension.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_8i2(p, lb1_new, ub1_new, lb2_new, ub2_new)

      INTEGER(kind=int_8), DIMENSION(:, :), POINTER      :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new

      INTEGER(KIND=int_8), PARAMETER                     :: zero = 0

      INTEGER(KIND=int_8), ALLOCATABLE, DIMENSION(:, :)  :: work


# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb1, lb1_old, ub1, ub1_old
# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb2, lb2_old, ub2, ub2_old
# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p)) THEN
# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1_old = LBOUND(p, 1)
# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1_old = UBOUND(p, 1)
# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1 = MAX(lb1_new, lb1_old)
# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1 = MIN(ub1_new, ub1_old)
# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2_old = LBOUND(p, 2)
# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2_old = UBOUND(p, 2)
# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2 = MAX(lb2_new, lb2_old)
# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2 = MIN(ub2_new, ub2_old)
# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ALLOCATE (work(lb1:ub1,lb2:ub2))
# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      work(lb1:ub1,lb2:ub2) = p(lb1:ub1,lb2:ub2)
# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (p)
# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new))
# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   p = zero
# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p) .AND. ALLOCATED(work)) THEN
# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      p(lb1:ub1,lb2:ub2) = work(lb1:ub1,lb2:ub2)
# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (work)
# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 328 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_8i2

! **************************************************************************************************
!> \brief (Re)Allocate an real vector with a new dimension.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_r1(p, lb1_new, ub1_new)

      REAL(KIND=dp), DIMENSION(:), POINTER               :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new

      REAL(KIND=dp), PARAMETER                           :: zero = 0.0_dp

      REAL(KIND=dp), ALLOCATABLE, DIMENSION(:)           :: work


# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb1, lb1_old, ub1, ub1_old
# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p)) THEN
# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1_old = LBOUND(p, 1)
# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1_old = UBOUND(p, 1)
# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1 = MAX(lb1_new, lb1_old)
# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1 = MIN(ub1_new, ub1_old)
# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ALLOCATE (work(lb1:ub1))
# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      work(lb1:ub1) = p(lb1:ub1)
# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (p)
# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   ALLOCATE (p(lb1_new:ub1_new))
# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   p = zero
# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p) .AND. ALLOCATED(work)) THEN
# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      p(lb1:ub1) = work(lb1:ub1)
# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (work)
# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 350 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_r1

! **************************************************************************************************
!> \brief (Re)Allocate an real array with new dimensions.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_r2(p, lb1_new, ub1_new, lb2_new, ub2_new)

      REAL(KIND=dp), DIMENSION(:, :), POINTER            :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new

      REAL(KIND=dp), PARAMETER                           :: zero = 0.0_dp

      REAL(KIND=dp), ALLOCATABLE, DIMENSION(:, :)        :: work


# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb1, lb1_old, ub1, ub1_old
# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb2, lb2_old, ub2, ub2_old
# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p)) THEN
# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1_old = LBOUND(p, 1)
# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1_old = UBOUND(p, 1)
# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1 = MAX(lb1_new, lb1_old)
# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1 = MIN(ub1_new, ub1_old)
# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2_old = LBOUND(p, 2)
# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2_old = UBOUND(p, 2)
# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2 = MAX(lb2_new, lb2_old)
# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2 = MIN(ub2_new, ub2_old)
# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ALLOCATE (work(lb1:ub1,lb2:ub2))
# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      work(lb1:ub1,lb2:ub2) = p(lb1:ub1,lb2:ub2)
# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (p)
# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new))
# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   p = zero
# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p) .AND. ALLOCATED(work)) THEN
# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      p(lb1:ub1,lb2:ub2) = work(lb1:ub1,lb2:ub2)
# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (work)
# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 374 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_r2

! **************************************************************************************************
!> \brief (Re)Allocate an real array 3D with new dimensions.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \param lb3_new ...
!> \param ub3_new ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_r3(p, lb1_new, ub1_new, lb2_new, ub2_new, lb3_new, ub3_new)

      REAL(KIND=dp), DIMENSION(:, :, :), POINTER         :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new, &
                                                            lb3_new, ub3_new

      REAL(KIND=dp), PARAMETER                           :: zero = 0.0_dp

      REAL(KIND=dp), ALLOCATABLE, DIMENSION(:, :, :)     :: work


# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb1, lb1_old, ub1, ub1_old
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb2, lb2_old, ub2, ub2_old
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb3, lb3_old, ub3, ub3_old
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p)) THEN
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1_old = LBOUND(p, 1)
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1_old = UBOUND(p, 1)
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1 = MAX(lb1_new, lb1_old)
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1 = MIN(ub1_new, ub1_old)
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2_old = LBOUND(p, 2)
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2_old = UBOUND(p, 2)
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2 = MAX(lb2_new, lb2_old)
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2 = MIN(ub2_new, ub2_old)
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb3_old = LBOUND(p, 3)
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub3_old = UBOUND(p, 3)
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb3 = MAX(lb3_new, lb3_old)
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub3 = MIN(ub3_new, ub3_old)
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ALLOCATE (work(lb1:ub1,lb2:ub2,lb3:ub3))
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      work(lb1:ub1,lb2:ub2,lb3:ub3) = p(lb1:ub1,lb2:ub2,lb3:ub3)
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (p)
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new,lb3_new:ub3_new))
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   p = zero
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p) .AND. ALLOCATED(work)) THEN
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      p(lb1:ub1,lb2:ub2,lb3:ub3) = work(lb1:ub1,lb2:ub2,lb3:ub3)
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (work)
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 401 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_r3

! **************************************************************************************************
!> \brief (Re)Allocate an real array 4D with new dimensions.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \param lb3_new ...
!> \param ub3_new ...
!> \param lb4_new ...
!> \param ub4_new ...
!> \par History
!>      none
!> \author Matthias Krack (04.10.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_r4(p, lb1_new, ub1_new, lb2_new, ub2_new, lb3_new, ub3_new, &
                            lb4_new, ub4_new)

      REAL(KIND=dp), DIMENSION(:, :, :, :), POINTER      :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new, &
                                                            lb3_new, ub3_new, lb4_new, ub4_new

      REAL(KIND=dp), PARAMETER                           :: zero = 0.0_dp

      REAL(KIND=dp), ALLOCATABLE, DIMENSION(:, :, :, :)  :: work


# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb1, lb1_old, ub1, ub1_old
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb2, lb2_old, ub2, ub2_old
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb3, lb3_old, ub3, ub3_old
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb4, lb4_old, ub4, ub4_old
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p)) THEN
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1_old = LBOUND(p, 1)
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1_old = UBOUND(p, 1)
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1 = MAX(lb1_new, lb1_old)
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1 = MIN(ub1_new, ub1_old)
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2_old = LBOUND(p, 2)
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2_old = UBOUND(p, 2)
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2 = MAX(lb2_new, lb2_old)
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2 = MIN(ub2_new, ub2_old)
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb3_old = LBOUND(p, 3)
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub3_old = UBOUND(p, 3)
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb3 = MAX(lb3_new, lb3_old)
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub3 = MIN(ub3_new, ub3_old)
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb4_old = LBOUND(p, 4)
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub4_old = UBOUND(p, 4)
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb4 = MAX(lb4_new, lb4_old)
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub4 = MIN(ub4_new, ub4_old)
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ALLOCATE (work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4))
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4) = p(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4)
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (p)
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new,lb3_new:ub3_new,lb4_new:ub4_new))
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   p = zero
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p) .AND. ALLOCATED(work)) THEN
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      p(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4) = work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4)
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (work)
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 431 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_r4

! **************************************************************************************************
!> \brief (Re)Allocate an real array 5D with new dimensions.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \param lb2_new ...
!> \param ub2_new ...
!> \param lb3_new ...
!> \param ub3_new ...
!> \param lb4_new ...
!> \param ub4_new ...
!> \param lb5_new ...
!> \param ub5_new ...
!> \par History
!>      none
!> \author Matthias Krack (04.10.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_r5(p, lb1_new, ub1_new, lb2_new, ub2_new, lb3_new, ub3_new, &
                            lb4_new, ub4_new, lb5_new, ub5_new)

      REAL(KIND=dp), DIMENSION(:, :, :, :, :), POINTER   :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new, lb2_new, ub2_new, &
                                                            lb3_new, ub3_new, lb4_new, ub4_new, &
                                                            lb5_new, ub5_new

      REAL(KIND=dp), PARAMETER                           :: zero = 0.0_dp

      REAL(KIND=dp), ALLOCATABLE, &
         DIMENSION(:, :, :, :, :)                        :: work


# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb1, lb1_old, ub1, ub1_old
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb2, lb2_old, ub2, ub2_old
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb3, lb3_old, ub3, ub3_old
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb4, lb4_old, ub4, ub4_old
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb5, lb5_old, ub5, ub5_old
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p)) THEN
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1_old = LBOUND(p, 1)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1_old = UBOUND(p, 1)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1 = MAX(lb1_new, lb1_old)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1 = MIN(ub1_new, ub1_old)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2_old = LBOUND(p, 2)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2_old = UBOUND(p, 2)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb2 = MAX(lb2_new, lb2_old)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub2 = MIN(ub2_new, ub2_old)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb3_old = LBOUND(p, 3)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub3_old = UBOUND(p, 3)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb3 = MAX(lb3_new, lb3_old)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub3 = MIN(ub3_new, ub3_old)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb4_old = LBOUND(p, 4)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub4_old = UBOUND(p, 4)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb4 = MAX(lb4_new, lb4_old)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub4 = MIN(ub4_new, ub4_old)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb5_old = LBOUND(p, 5)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub5_old = UBOUND(p, 5)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb5 = MAX(lb5_new, lb5_old)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub5 = MIN(ub5_new, ub5_old)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ALLOCATE (work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4,lb5:ub5))
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4,lb5:ub5) = p(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4,lb5:ub5)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (p)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   ALLOCATE (p(lb1_new:ub1_new,lb2_new:ub2_new,lb3_new:ub3_new,lb4_new:ub4_new,lb5_new:ub5_new))
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   p = zero
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p) .AND. ALLOCATED(work)) THEN
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      p(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4,lb5:ub5) = work(lb1:ub1,lb2:ub2,lb3:ub3,lb4:ub4,lb5:ub5)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (work)
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 465 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_r5

! **************************************************************************************************
!> \brief (Re)Allocate a logical vector with a new dimension.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
! **************************************************************************************************
   SUBROUTINE reallocate_l1(p, lb1_new, ub1_new)

      LOGICAL, DIMENSION(:), POINTER                     :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new

      LOGICAL, PARAMETER                                 :: zero = .FALSE.

      LOGICAL, ALLOCATABLE, DIMENSION(:)                 :: work


# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb1, lb1_old, ub1, ub1_old
# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p)) THEN
# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1_old = LBOUND(p, 1)
# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1_old = UBOUND(p, 1)
# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1 = MAX(lb1_new, lb1_old)
# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1 = MIN(ub1_new, ub1_old)
# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ALLOCATE (work(lb1:ub1))
# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      work(lb1:ub1) = p(lb1:ub1)
# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (p)
# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   ALLOCATE (p(lb1_new:ub1_new))
# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   p = zero
# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p) .AND. ALLOCATED(work)) THEN
# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      p(lb1:ub1) = work(lb1:ub1)
# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (work)
# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 487 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_l1

! **************************************************************************************************
!> \brief (Re)Allocate a vector of string variables
!> \param p_short ...
!> \param lb_new ...
!> \param ub_new ...
!> \param p_long ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
!> \note
!>      (Maybe outdated) This routine doesnt work on SUN/Solaris!!!
!>                       It should probably not be used. (Thomas Chassaing)
! **************************************************************************************************
   SUBROUTINE reallocate_s1(p_short, lb_new, ub_new, p_long)

      CHARACTER(LEN=default_string_length), &
         DIMENSION(:), OPTIONAL, POINTER                 :: p_short
      INTEGER, INTENT(IN)                                :: lb_new, ub_new
      CHARACTER(LEN=default_path_length), DIMENSION(:), &
         OPTIONAL, POINTER                               :: p_long

      IF (PRESENT(p_short)) THEN
         CALL reallocate_ss1(p_short, lb_new, ub_new)
      ELSE IF (PRESENT(p_long)) THEN
         CALL reallocate_ls1(p_long, lb_new, ub_new)
      ELSE
         CPABORT("At least one of the two optional arguments is required")
      END IF

   END SUBROUTINE reallocate_s1

! **************************************************************************************************
!> \brief (Re)Allocate a vector of string variables of default string length.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \par History
!>      none
!> \author Thomas Chassaing
!> \author Matthias Krack (18.07.2002,MK)
!> \note
!>      (Maybe outdated) This routine doesnt work on SUN/Solaris!!!
!>                       It should probably not be used. (Thomas Chassaing)
! **************************************************************************************************
   SUBROUTINE reallocate_ss1(p, lb1_new, ub1_new)

      CHARACTER(LEN=default_string_length), &
         DIMENSION(:), POINTER                           :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new

      CHARACTER(LEN=default_string_length), PARAMETER    :: zero = ""

      CHARACTER(LEN=default_string_length), &
         ALLOCATABLE, DIMENSION(:)                       :: work


# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb1, lb1_old, ub1, ub1_old
# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p)) THEN
# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1_old = LBOUND(p, 1)
# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1_old = UBOUND(p, 1)
# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1 = MAX(lb1_new, lb1_old)
# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1 = MIN(ub1_new, ub1_old)
# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ALLOCATE (work(lb1:ub1))
# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      work(lb1:ub1) = p(lb1:ub1)
# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (p)
# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   ALLOCATE (p(lb1_new:ub1_new))
# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   p = zero
# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p) .AND. ALLOCATED(work)) THEN
# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      p(lb1:ub1) = work(lb1:ub1)
# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (work)
# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 546 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_ss1

! **************************************************************************************************
!> \brief (Re)Allocate a vector of string variables of default path length.
!> \param p ...
!> \param lb1_new ...
!> \param ub1_new ...
!> \par History
!>      none
!> \author Matthias Krack (18.07.2002,MK)
!> \note
!>      (Maybe outdated) This routine doesnt work on SUN/Solaris!!!
!>                       It should probably not be used. (Thomas Chassaing)
! **************************************************************************************************
   SUBROUTINE reallocate_ls1(p, lb1_new, ub1_new)

      CHARACTER(LEN=default_path_length), DIMENSION(:), &
         POINTER                                         :: p
      INTEGER, INTENT(IN)                                :: lb1_new, ub1_new

      CHARACTER(LEN=default_path_length), PARAMETER      :: zero = ""

      CHARACTER(LEN=default_path_length), ALLOCATABLE, &
         DIMENSION(:)                                    :: work


# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   INTEGER :: lb1, lb1_old, ub1, ub1_old
# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p)) THEN
# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1_old = LBOUND(p, 1)
# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1_old = UBOUND(p, 1)
# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      lb1 = MAX(lb1_new, lb1_old)
# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ub1 = MIN(ub1_new, ub1_old)
# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      ALLOCATE (work(lb1:ub1))
# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      work(lb1:ub1) = p(lb1:ub1)
# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (p)
# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   ALLOCATE (p(lb1_new:ub1_new))
# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   p = zero
# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"

# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   IF (ASSOCIATED(p) .AND. ALLOCATED(work)) THEN
# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      p(lb1:ub1) = work(lb1:ub1)
# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
      DEALLOCATE (work)
# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"
   END IF
# 573 "/builddir/build/BUILD/cp2k-dbf7a770d1541ba72a4652ee218de80c0484db2d/src/common/memory_utilities.F"


   END SUBROUTINE reallocate_ls1

END MODULE memory_utilities
